****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatables.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 11/14/2016
* Version: 1.3
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output tables for type 1 and type 2 cohorts.                                        
*   
*  Program inputs:    
*	-infolder.&cohortfile.
*	-infolder.&COMORBFILE.
*	-infolder.&UTILFILE.
*	-DPLocal.&RUNID._mstr
*	-DPLOCAL.&RUNID._DenomCounts 
*                                     
*  Program outputs:  
*	-DPLOCAL.&RUNID._NumCounts
*	-MSOC.&RUNID._t1_CIDA
*	-MSOC.&RUNID._t2_CIDA 
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.3	  11/14/16    AP		Added two new output tables: t1/t2_cida_zip3 and t1/t2_cida_state
*
***************************************************************************************************;


%macro ms_cidatables();

%put =====> MACRO CALLED: ms_cidatables v1.3;

    *creating age group dataset;
    data _age;
    numgrp = countw("&AGESTRAT."," ") ;
    do i=1 to numgrp;
        AgeGroup=scan("&AGESTRAT",i," ");
        AgeGroupSort=i;
        output;
    end;
    keep AgeGroup AgeGroupSort;
    run;


    /****************************************************************/
    /* Squaring (or making sure all possible values are represented */
    /****************************************************************/
    data _AgeGroupYearMonthSex;
    set infolder.&cohortfile.;
	if upcase(Type1)="Y" then Type=1;
	else Type=2;
    format AgeGroup $9.;
    numgrp = countw("&AGESTRAT."," ") ;
    NumDisp=0; TotRxSup=0; TotRxAmt=0; NumEvents=0; tte=0;
    do i=1 to numgrp;
        AgeGroup=scan("&AGESTRAT",i," ");
        AgeGroupSort=i;
        do per=1 to &nummonths.;
            *create year month variables;
            date=intnx('month',&startfollowup.,per-1,'begin');
            month=month(date);
            year=year(date);
            sex="F";output;
            sex="M";output;
            sex="U";output;
            sex="A";output;
        end;
    end;
    keep group Type AgeGroup AgeGroupSort sex year month NumDisp TotRxSup TotRxAmt NumEvents tte;
    run;


    %let ccielixgrp=;
    %let ccielixgrp2='' as ccielixgrp  length=10,;
	%if %str(&COMORBFILE.) ne %str() or %length(&COMORBFILE.) > 0. %then %do;	
        data _ccielixgrp;
        set infolder.&COMORBFILE.;
        format ccielixgrp $10.;
        ccielixgrp="";output;
        numgrp = countw(ccigroup," ");
        do i=1 to numgrp;
            ccielixgrp=scan(ccigroup,i," ");
            output;
        end;
        keep group ccielixgrp;
        run;

        proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSex as
        select base.*,
               com.ccielixgrp
        from  _AgeGroupYearMonthSex as base,
              _ccielixgrp as com
        where base.group = com.group;
        quit;

        %let ccielixgrp=ccielixgrp;
        %let ccielixgrp2=ccielixgrp,;
    %END;

    %let NumVisits=;
    %let NumVisits2='' as NumVisits  length=10,;;
    %if %str(&UTILFILE.) ne %str() or %length(&UTILFILE.) > 0. %then %do;	
        data _NumVisits(rename=csstrt=NumVisits);
        set infolder.&UTILFILE.;
        format csstrt $10.;
        csstrt="";output;
        numgrp = countw(csstrat," ") ;
        do i=1 to numgrp;
            csstrt=scan(csstrat,i," ");
            output;
        end;
        keep group csstrt;
        run;

        proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSex as
        select base.*,
               com.NumVisits
        from  _AgeGroupYearMonthSex as base,
              _NumVisits as com
        where base.group = com.group;
        quit;

        %let NumVisits=NumVisits;
        %let NumVisits2=NumVisits,;
    %END;
	%if %str(&ZIPFILE.) ne %str() or %length(&ZIPFILE.) > 0. %then %do;	
		/*Create temporary input file representing all group*zip combinations*/
		proc sql noprint;
			create table _ziplkp as
			select	zip.*
				,	chrt.group
			from infolder.&zipfile. as zip,
			infolder.&cohortfile. as chrt;
		quit;

		proc sql noprint;
			create table _zipcodes as 
			select distinct group 
				, substr(zip,1,3) as zip3 format=$7. length=7
			from _ziplkp
			order by group;

			create table _statecodes as 
			select distinct group 
				, statecode as state  format=$7. length=7
			from _ziplkp
			order by group;
		quit;

		/*Add Missing and Invalid rows*/
 		data _zipcodes;
			set _zipcodes;
			by group;
        	format zip3 $7.;
			output;

			if last.group then do;;
	         	zip3='Missing';
			  	output;
				zip3='Invalid';
				output;
     		end;
        	keep group zip3;
       run;

 		data _statecodes;
			set _statecodes;
			by group;
        	format state $7.;
			output;

			if last.group then do;;
	         	state='Missing';
			  	output;
				state='Invalid';
				output;
     		end;
        	keep group state;
       run;

	   /*drop month to prevent unnecessary calculations*/
		proc sort data=_AgeGroupYearMonthSex(drop=month) nodup out=_AgeGroupYearMonthSexZIP3; 
			by _all_; 
		run;
		proc sort data=_AgeGroupYearMonthSex(drop=month) nodup out=_AgeGroupYearMonthSexSTATE; 
			by _all_; 
		run;

		proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSexZIP3 as
        select base.*,
               com.zip3
        from  _AgeGroupYearMonthSexZIP3 as base,
              _zipcodes as com
        where base.group = com.group;
        quit;

		proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSexSTATE as
        select base.*,
               com.state
        from  _AgeGroupYearMonthSexSTATE as base,
              _statecodes as com
        where base.group = com.group;
        quit;

		/*Add uncertain*/
		proc sort data=infolder.&cohortfile. out=cohortsorted;
			by group;
		run;

		data _uncertain;
			set cohortsorted (keep=group);
			by group;

			if last.group then do;;
	         	zip_uncertain='N';
			  	output;
				zip_uncertain='Y';
				output;
     		end;
        	keep group zip_uncertain;
       run;
	  
		proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSexZIP3 as
        select base.*,
               com.zip_uncertain
        from  _AgeGroupYearMonthSexZIP3 as base,
              _uncertain as com
        where base.group = com.group;
        quit;

		proc sql noprint  undo_policy=none;
        create table _AgeGroupYearMonthSexSTATE as
        select base.*,
               com.zip_uncertain
        from  _AgeGroupYearMonthSexSTATE as base,
              _uncertain as com
        where base.group = com.group;
        quit;

	   	proc datasets nowarn noprint lib=work; 
			delete cohortsorted _uncertain _zipcodes _statecodes;
		quit;
	%END;

  
    /**********************/
    /* Compute Numerators */
    /**********************/
%macro createdata(table=);

  proc sql noprint;
    create table _mstr&table. as 
    select mstr.*,
           age.AgeGroupSort
    from  DPLocal.&RUNID._mstr as mstr left join
          _age as age
    on age.AgeGroup=mstr.AgeGroup;
    quit;

*Set ccielixgrp and NumVisits to missing when their values are outside the required groups;
%ISDATA(dataset=_mstr&table.);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	%if %str(&COMORBFILE.) ne %str() or %length(&COMORBFILE.) > 0. %then %do;
		proc sql noprint;
			update _mstr&table. set &ccielixgrp. = "" where &ccielixgrp. not in 
				(select distinct &ccielixgrp. from _ccielixgrp);
		quit;
	%end;

	%if %str(&UTILFILE.) ne %str() or %length(&UTILFILE.) > 0. %then %do;	
		proc sql noprint;
			update _mstr&table. set &NumVisits. = "" where &NumVisits. not in 
				(select distinct &NumVisits. from _numvisits);
		quit;
	%end;
%END;

    data _mstr&table.;
    set _mstr&table.(in=a)
        _AgeGroupYearMonthSex&table.(in=b);
    if a then do;
		%if %str(&table.) = %str() %then %do;
        month=month(indexdt);
		%end;
        patient=1;
        if NumEvents>0 then HadEvent=1;
    end;
    else do;
        patient=0;    
        HadEvent=0; 
    end;
    run;
%mend;

%createdata(table=);

%if %index(&geog.,ZIP3) > 0 %then %do;
%createdata(table=ZIP3);
%end;
%if %index(&geog.,STATE) > 0 %then %do;
%createdata(table=STATE);
%end;


    %macro createNumTableEntries(table=,class=,outfile=,level=);

        *One record per patient for the class stratification;
        proc means data=_mstr&table. nway noprint missing;
        var Patient NumDisp TotRxSup TotRxAmt HadEvent NumEvents tte;
        class group type PatId &class.;
        id patient;
        output out=_num0(drop=_:) max(Patient)=Npts
                                  sum(Patient)=Episodes           
                                  sum(NumDisp)=Dispensings         
                                  sum(TotRxSup)=DaySupp
    					          sum(TotRxAmt)=AmtSupp
    						      sum(HadEvent)=Eps_wEvents
                                  sum(NumEvents)=All_Events
    							  sum(tte)=tte;             
        run;

        *One record per patient for the class stratification;
        proc means data=_num0 missing nway noprint;
        var Npts Episodes Dispensings DaySupp AmtSupp Eps_wEvents All_Events tte; ;
        class group type &class.;
        output out=&outfile.(drop=_:) sum=;             
        run;
	
	        *Store patient nums to MSOC;
	        %IF %length(&class.)=0 %THEN %DO;
	            data DPLOCAL.&RUNID._NumCounts&TABLE.;
	            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
	            set &outfile.;
	            format level $3.;
	            Level="&level.";
	            run;
	        %END;
	        %ELSE %DO;
	            data DPLOCAL.&RUNID._NumCounts&TABLE.;
	            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
	            set DPLOCAL.&RUNID._NumCounts&TABLE.
	                &outfile.(in=a);
	                if a then Level="&level.";
	            run;
	        %END;
    %mend;

    *Overall;
    %createNumTableEntries(table=,class=,outfile=_ALL,level=000);

    *Base Tables;
    %createNumTableEntries(table=,class=year,outfile=_Y,Level=001);
    %createNumTableEntries(table=,class=Sex,outfile=_S,Level=002);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort,outfile=_AG,Level=003);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort,outfile=_SAG,Level=004);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year,outfile=_SAGY,Level=005);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year month,outfile=_SAGYM,Level=006);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year,outfile=_AGY,Level=007);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year month,outfile=_AGYM,Level=008);
    %createNumTableEntries(table=,class=Sex year,outfile=_SY,Level=009);
    %createNumTableEntries(table=,class=Sex year month,outfile=_SYM,Level=010);
	%createNumTableEntries(table=,class=year month,outfile=_YM,Level=011);

    *Util tables Tables;
    %if %length(&NumVisits.)>0 %then %do;
    %createNumTableEntries(table=,class=year &NumVisits.,outfile=_Y_UT,Level=101);
    %createNumTableEntries(table=,class=Sex &NumVisits.,outfile=_S_UT,Level=102);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort &NumVisits.,outfile=_AG_UT,Level=103);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort &NumVisits.,outfile=_SAG_UT,Level=104);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year &NumVisits.,outfile=_SAGY_UT,Level=105);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year month &NumVisits.,outfile=_SAGYM_UT,Level=106);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year &NumVisits.,outfile=_AGY_UT,Level=107);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year month &NumVisits.,outfile=_AGYM_UT,Level=108);
    %createNumTableEntries(table=,class=Sex year &NumVisits.,outfile=_SY_UT,Level=109);
    %createNumTableEntries(table=,class=Sex year month &NumVisits.,outfile=_SYM_UT,Level=110);
	%createNumTableEntries(table=,class=year month &NumVisits.,outfile=_YM_UT,Level=111);
    %createNumTableEntries(table=,class=&NumVisits.,outfile=_UT,Level=112);
    %end;

    %if %length(&ccielixgrp.)>0 %then %do;
    %createNumTableEntries(table=,class=year &ccielixgrp.,outfile=_Y_CE,Level=201);
    %createNumTableEntries(table=,class=Sex &ccielixgrp.,outfile=_S_CE,Level=202);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort &ccielixgrp.,outfile=_AG_CE,Level=203);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort &ccielixgrp.,outfile=_SAG_CE,Level=204);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year &ccielixgrp.,outfile=_SAGY_CE,Level=205);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year month &ccielixgrp.,outfile=_SAGYM_CE,Level=206);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year &ccielixgrp.,outfile=_AGY_CE,Level=207);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year month &ccielixgrp.,outfile=_AGYM_CE,Level=208);
    %createNumTableEntries(table=,class=Sex year &ccielixgrp.,outfile=_SY_CE,Level=209);
    %createNumTableEntries(table=,class=Sex year month &ccielixgrp.,outfile=_SYM_CE,Level=210);
	%createNumTableEntries(table=,class=year month &ccielixgrp.,outfile=_YM_CE,Level=211);
    %createNumTableEntries(table=,class=&ccielixgrp.,outfile=_CE,Level=212);
    %end;

    %if %length(&NumVisits.)>0 & %length(&ccielixgrp.)>0 %then %do;
    %createNumTableEntries(table=,class=year &NumVisits. &ccielixgrp.,outfile=_Y_UT_CE,Level=301);
    %createNumTableEntries(table=,class=Sex &NumVisits. &ccielixgrp.,outfile=_S_UT_CE,Level=302);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort &NumVisits. &ccielixgrp.,outfile=_AG_UT_CE,Level=303);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort &NumVisits. &ccielixgrp.,outfile=_SAG_UT_CE,Level=304);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year &NumVisits. &ccielixgrp.,outfile=_SAGY_UT_CE,Level=305);
    %createNumTableEntries(table=,class=Sex AgeGroup agegroupsort year month &NumVisits. &ccielixgrp.,outfile=_SAGYM_UT_CE,Level=306);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year &NumVisits. &ccielixgrp.,outfile=_AGY_UT_CE,Level=307);
    %createNumTableEntries(table=,class=AgeGroup agegroupsort year month &NumVisits. &ccielixgrp.,outfile=_AGYM_UT_CE,Level=308);
    %createNumTableEntries(table=,class=Sex year &NumVisits. &ccielixgrp.,outfile=_SY_UT_CE,Level=309);
    %createNumTableEntries(table=,class=Sex year month &NumVisits. &ccielixgrp.,outfile=_SYM_UT_CE,Level=310);
	%createNumTableEntries(table=,class=year month &NumVisits. &ccielixgrp.,outfile=_YM_UT_CE,Level=311);
    %createNumTableEntries(table=,class=&NumVisits. &ccielixgrp.,outfile=_UT_CE,Level=312);
    %end;

	%if %index(&geog.,ZIP3) > 0 %then %do;
 	%createNumTableEntries(table=ZIP3,class=,outfile=_ALL,level=000);
	%createNumTableEntries(table=ZIP3,class=zip3,outfile=_Z,Level=020);
	%createNumTableEntries(table=ZIP3,class=zip3 zip_uncertain,outfile=_Z_UN,Level=021);
	%createNumTableEntries(table=ZIP3,class=zip3 Sex,outfile=_Z_S,Level=022);
	%createNumTableEntries(table=ZIP3,class=zip3 Sex zip_uncertain,outfile=_Z_S_UN,Level=023);
	%createNumTableEntries(table=ZIP3,class=zip3 AgeGroup agegroupsort,outfile=_Z_AG,Level=024);
	%createNumTableEntries(table=ZIP3,class=zip3 AgeGroup agegroupsort zip_uncertain ,outfile=_Z_AG_UN,Level=025);
	%createNumTableEntries(table=ZIP3,class=zip3 year,outfile=_Z_Y,Level=026);
	%createNumTableEntries(table=ZIP3,class=zip3 year zip_uncertain,outfile=_Z_Y_UN,Level=027);
	%end;

	%if %index(&geog.,STATE) > 0 %then %do;
 	%createNumTableEntries(table=STATE,class=,outfile=_ALL,level=000);
	%createNumTableEntries(table=STATE,class=STATE,outfile=_S,Level=030);
	%createNumTableEntries(table=STATE,class=STATE zip_uncertain,outfile=_S_UN,Level=031);
	%createNumTableEntries(table=STATE,class=STATE Sex,outfile=_S_S,Level=032);
	%createNumTableEntries(table=STATE,class=STATE Sex zip_uncertain,outfile=_S_S_UN,Level=033);
	%createNumTableEntries(table=STATE,class=STATE AgeGroup agegroupsort,outfile=_S_AG,Level=034);
	%createNumTableEntries(table=STATE,class=STATE AgeGroup agegroupsort zip_uncertain ,outfile=_S_AG_UN,Level=035);
	%createNumTableEntries(table=STATE,class=STATE year,outfile=_S_Y,Level=036);
	%createNumTableEntries(table=STATE,class=STATE year zip_uncertain,outfile=_S_Y_UN,Level=037);
	%end;


*The table to be generated in MSOC will be the result of a merge between numerators and donominators;
%macro num_denom_merge(table=, byvars=);
	proc sort data=DPLOCAL.&RUNID._NumCounts&table.;
	by &byvars. ;
	run; 

	proc sort data=DPLOCAL.&RUNID._DenomCounts&table.;
	by &byvars.;
	run; 

	data _t1_CIDA&table.
		 _t2_CIDA&table.;
	merge DPLOCAL.&RUNID._NumCounts&table. (in=a)
		  DPLOCAL.&RUNID._DenomCounts&table. (in=b);
	by &byvars.;

	if not b and level=:"0" then do;
	DenNumPts=0;
	DenNumMemDays=0;
	end;

	if b and level=:"0" and missing(DenNumPts) then do;
	DenNumPts=0;
	DenNumMemDays=0;
	end;

	if type=1 then output _t1_CIDA&table.;
	else if type=2 then output _t2_CIDA&table.;
	run;
%mend;
%num_denom_merge(table=, byvars= group Level sex agegroupsort year month);

%if %index(&geog.,ZIP3) > 0 %then %do;
%num_denom_merge(table=ZIP3, byvars=group Level zip3 zip_uncertain sex agegroupsort year);
%end;

%if %index(&geog.,STATE) > 0 %then %do;
%num_denom_merge(table=STATE, byvars=group Level state zip_uncertain sex agegroupsort year);
%end;


* Output to MSOC if non empty;
%ISDATA(dataset=_t1_CIDA);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t1_CIDA as
	select 	group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		month,
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays
    from _t1_CIDA;
	quit;
%END;

%ISDATA(dataset=_t2_CIDA);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t2_CIDA as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		month,
    		&NumVisits2.
    		&ccielixgrp2.
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays 
    from _t2_CIDA;
	quit;
%END;


%ISDATA(dataset=_t1_CIDAzip3);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t1_CIDA_zip3 as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		zip3,
			zip_uncertain,
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays
    from _t1_CIDAzip3;
	quit;
%END;
%ISDATA(dataset=_t1_CIDAstate);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t1_CIDA_state as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		state,
			zip_uncertain,
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays
    from _t1_CIDAstate;
	quit;
%END;

%ISDATA(dataset=_t2_CIDAzip3);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t2_CIDA_zip3 as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		zip3,
			zip_uncertain,
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays
    from _t2_CIDAzip3;
	quit;
%END;
%ISDATA(dataset=_t2_CIDAstate);
%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
	create table MSOC.&RUNID._t2_CIDA_state as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		state,
			zip_uncertain,
    		Npts,
    		Episodes,
    		Dispensings,
    		DaySupp,
    		AmtSupp,
    		Eps_wEvents,
    		All_Events,
    		tte,
    		DenNumPts,
    		DenNumMemDays
    from _t2_CIDAstate;
	quit;
%END;
	proc datasets library=work nolist nowarn;
	delete _: &RUNID._mstr;
	quit;

%put NOTE: ******** END OF MACRO: ms_cidatables  v1.3 ********;

%mend ms_cidatables;
