****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest3.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 11/30/2016
* Version: 1.6
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output tables for type 3 cohort.                                        
*   
*  Program inputs:   
*	-infolder.&TYPE3FILE.
*	-infolder.&T3METADATA.
*	-DPLocal.&RUNID._mstr 
*                                     
*  Program outputs:  
*	-DPLOCAL.&RUNID._NumCounts
*	-MSOC.&RUNID._t3_CIDA
*	-msoc.&RUNID._Metadata_for_time_period_&PERIODIDSTART. 
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1      02/23/14	    DM	    Changed the way EPISODES_ANALYSIS and EPISODES_EXPOSURE are
*                                   calculated
*
*    1.2      03/18/15      DM      Changed the way ExpPeriodEndDt is calculated	  
*									
*	 1.3      06/30/16      DM      Added tables with Time to censor data
*     
*	 1.4      07/13/16      DM      Fix for JIRA #QCI-178
*
*    1.5      11/07/16      DM      Fix for JIRA #QRP-279
*
*    1.6	  11/30/16		AP		Fix for JIRA #QRP-282
***************************************************************************************************;


%macro ms_cidatablest3();

%put =====> MACRO CALLED: ms_cidatablest3 v1.6;

    *creating age group dataset;
    data _age;
    numgrp = countw("&AGESTRAT."," ") ;
    do i=1 to numgrp;
        AgeGroup=scan("&AGESTRAT",i," ");
        AgeGroupSort=i;
        output;
    end;
    keep AgeGroup AgeGroupSort;
    run;

	*Get parameters necessary to compute ExpPeriodEndDt;
	data _null_;
    dset=open("infolder.&COHORTFILE.");    
	call symput ('EnrDaysAft',put(varnum(dset,'EnrDaysAft'),best.));
    run;
	
	data _CohortInfo;
	set infolder.&COHORTFILE.;
	if &EnrDaysAft.=0 then EnrDaysAft=0;
	keep group EnrDaysAft;
	run;	

	proc sort nodupkey data=_CohortInfo;
	by group;
	run;

	proc sort data=infolder.&TYPE3FILE. out= _Type3Info;
    by group;
    run;	

	data _Type3Info;
	merge _Type3Info _CohortInfo;
	by group;
	CondTo=0;	
	run;

	%ISDATA(dataset=infolder.&INCLUSIONCODES.);
    %IF %EVAL(&NOBS.>=1) %THEN %DO;			
		proc sort data = infolder.&INCLUSIONCODES. out= _CondsTo(keep=Group CondTo);
		by group descending CondTo;
		where Condinclusion=0 or subcondinclusion = 0;
		run;

		proc means data=_CondsTo nway noprint;
		class group;
		var CondTo;
		output out=_CondsTo (drop=_:) max=;
		run;

		data _Type3Info;
		merge _Type3Info _CondsTo;
		by group;
		run;	
	%END;	

    data _metadata;
    set _Type3Info;
    format T3SURVSTARTDATE T3ENDOFUPDATE ExpPeriodStartDt ExpPeriodEndDt date9.;

    TimePeriod=&PERIODIDSTART.;
    RequiredFUPPer=max(0,T3RiskTo,T3CtrlTo,EnrDaysAft,CondTo);

    ExpPeriodStartDt=T3SURVSTARTDATE;
    ExpPeriodEndDt=INTNX('Month', T3ENDOFUPDATE-RequiredFUPPer,0, 'begin')-1;

    keep group TimePeriod ExpPeriodStartDt ExpPeriodEndDt T3SURVSTARTDATE T3ENDOFUPDATE;
    run;

    proc sort data=_metadata;
    by group;
    run;

    %if &PERIODIDSTART. > 1 %then %do;
		data _IncrDates(drop=ExpPeriodEndDt T3ENDOFUPDATE); 
		format ExpPeriodStartDt date9.;
        set infolder.&T3METADATA.;  
		ExpPeriodStartDt=INTNX('Month', ExpPeriodEndDt+1, 0, 'begin');
		run;

        proc sort data=_IncrDates;
        by group;
        run;

        data _metadata;
        merge _metadata(drop=ExpPeriodStartDt)
              _IncrDates;
        by group;
		TimePeriod=&PERIODIDSTART.;
        *ExpPeriodStartDt=&ExpPeriodStartDt.;
		keep group TimePeriod ExpPeriodStartDt ExpPeriodEndDt T3SURVSTARTDATE T3ENDOFUPDATE;
        run;

    %end;

	data msoc.&runid._Metadata_for_time_period_&PERIODIDSTART.;
	retain MSReqID RunID GROUP TimePeriod T3SURVSTARTDATE ExpPeriodStartDt ExpPeriodEndDt T3ENDOFUPDATE;
	set _metadata;
	MSReqID="&MSREQID.";
	RunID="&RUNID.";		
	run;

	data _null_;
    format startfollowup enddate date9.;
    startfollowup=&MinExpPeriodStartDt.;
    enddate=&MaxExpPeriodEndDt.;
    call symputx("startfollowup",startfollowup);
    call symputx("enddate",enddate);
    nummonths=intck('month',startfollowup,enddate)+1;
    call symputx("nummonths",nummonths);
    run;
    %put &startfollowup. &enddate. &nummonths.;

    proc sort data=infolder.&cohortfile. out=_cohortfile;
    by group;
    run;
    proc sort data=infolder.&TYPE3FILE. out=_TYPE3FILE;
    by group;
    run;

%macro ms_createTablesT3NoCensor();

	/****************************************************************/
    /* Squaring (or making sure all possible values are represented */
    /****************************************************************/
    data _AgeGroupYearMonthSex;
    merge _cohortfile
          _TYPE3FILE(keep=group T3CtrlFrom T3CtrlTo T3RiskFrom T3RiskTo);
    by group;
    Type=3;
    format AgeGroup $9.;
    numgrp = countw("&AGESTRAT."," ") ;

    *Sumarization Vars;
	NPTS_EXPOSURE=0;
	NPTS_CENSOR_ELIG=0; 
	NPTS_CENSOR_DTH=0;
	NPTS_CENSOR_NOEVENTS=0;
	NPTS_ANALYSIS=0;
	EPISODES_EXPOSURE=0;
	EPISODES_ANALYSIS=0;
	EVENTS_ANALYSIS_RISK=0;
	EVENTS_ANALYSIS_CTRL=0;
	MINDAYS_EVENT_ANALYSIS=0;
	MAXDAYS_EVENT_ANALYSIS=0;
	MINDAYS_POSTENR_EXPOSURE=0;
	MAXDAYS_POSTENR_EXPOSURE=0;
	EPISODES_CENSOR_ELIG=0;
	EPISODES_CENSOR_DTH=0;
	EPISODES_CENSOR_NOEVENTS=0;

    do i=1 to numgrp;
        AgeGroup=scan("&AGESTRAT",i," ");
        AgeGroupSort=i;
        do per=1 to &nummonths.;
            *create year month variables;
            date=intnx('month',&startfollowup.,per-1,'begin');
            month=month(date);
            year=year(date);
            do TTE_VALUE=T3CtrlFrom to T3CtrlTo;
                    sex="F";output;
                    sex="M";output;
                    sex="U";output;
                    sex="A";output;
            end;
            do TTE_VALUE=T3RiskFrom to T3RiskTo;
                    sex="F";output;
                    sex="M";output;
                    sex="U";output;
                    sex="A";output;
            end;
        end;
    end;
    keep group Type AgeGroup AgeGroupSort sex year month NPTS_: EPISODES_: EVENTS_: MINDAYS_: MAXDAYS_: TTE_VALUE;
    run;

    proc sql noprint;
    create table _mstr as 
    select mstr.*,
           age.AgeGroupSort
    from  DPLocal.&RUNID._mstr as mstr left join
          _age as age
    on age.AgeGroup=mstr.AgeGroup;
    quit;

    /**********************/
    /* Compute Numerators */
    /**********************/

    data _mstr;
    set _mstr(in=a)
        _AgeGroupYearMonthSex(in=b);
    if a then do;
        month=month(indexdt);
        NPTS_EXPOSURE=1;
        NPTS_CENSOR_ELIG=CensorEnrol; 
        NPTS_CENSOR_DTH=CensorDeath;
        NPTS_ANALYSIS=AnalysisCohort;
		EPISODES_CENSOR_ELIG=CensorEnrol;
		EPISODES_CENSOR_DTH=CensorDeath;		
		EPISODES_EXPOSURE=Pt_Exposure_Episode_Num>0;
		EPISODES_ANALYSIS=(Pt_Analysis_Episode_Num>0 and WinOfevent ne "");
        EVENTS_ANALYSIS_RISK=(WinOfEvent="RISK");
        EVENTS_ANALYSIS_CTRL=(WinOfEvent="CTRL");

        NPTS_CENSOR_NOEVENTS=max(0,AnalysisCohort,CensorDeath,CensorEnrol,EVENTS_ANALYSIS_RISK,EVENTS_ANALYSIS_CTRL)=0;
		EPISODES_CENSOR_NOEVENTS=NPTS_CENSOR_NOEVENTS;

        TTE_VALUE=DAYS_FROM_EXPOS_TO_EVENT;

		patient=1;
		/*already calculated in cidanum*/
		/*      
		        Pt_Exposure_Episode_Num
		        Pt_Analysis_Episode_Num
		        DAYS_FROM_EXPOS_TO_RISK_EVENT;
		        DAYS_FROM_EXPOS_TO_CTRL_EVENT;
		        DAYS_FROM_EXPOS_TO_EVENT;*/

    end;
    run;

    %macro createNumTableEntries(class=,outfile=,level=);

        *One record per patient for the class stratification;
        proc means data=_mstr nway noprint missing;
        var NPTS_EXPOSURE
            NPTS_CENSOR_ELIG 
            NPTS_CENSOR_DTH
            NPTS_ANALYSIS
            EPISODES_EXPOSURE
            EPISODES_ANALYSIS
            EVENTS_ANALYSIS_RISK
            EVENTS_ANALYSIS_CTRL
            NPTS_CENSOR_NOEVENTS
            POSTENR_EXPOSURE
			DAYS_FROM_EXPOS_TO_EVENT;

        class group type PatId &class.;
        id patient;
        output out=_num0(drop=_:) max(NPTS_EXPOSURE)=NPTS_EXPOSURE
                                  max(NPTS_CENSOR_ELIG)=NPTS_CENSOR_ELIG 
                                  max(NPTS_CENSOR_DTH)=NPTS_CENSOR_DTH
                                  max(NPTS_ANALYSIS)=NPTS_ANALYSIS
                                  sum(EPISODES_EXPOSURE)=EPISODES_EXPOSURE
                                  sum(EPISODES_ANALYSIS)=EPISODES_ANALYSIS
                                  sum(EVENTS_ANALYSIS_RISK)=EVENTS_ANALYSIS_RISK
                                  sum(EVENTS_ANALYSIS_CTRL)=EVENTS_ANALYSIS_CTRL
                                  max(NPTS_CENSOR_NOEVENTS)=NPTS_CENSOR_NOEVENTS
                                  min(POSTENR_EXPOSURE)=MINDAYS_POSTENR_EXPOSURE
                                  max(POSTENR_EXPOSURE)=MAXDAYS_POSTENR_EXPOSURE
                                  min(DAYS_FROM_EXPOS_TO_EVENT)=MINDAYS_EVENT_ANALYSIS
                                  max(DAYS_FROM_EXPOS_TO_EVENT)=MAXDAYS_EVENT_ANALYSIS;
        run;

        *One record per patient for the class stratification;
        proc means data=_num0 missing nway noprint;
        var NPTS_EXPOSURE
            NPTS_CENSOR_ELIG 
            NPTS_CENSOR_DTH
            NPTS_ANALYSIS
            EPISODES_EXPOSURE
            EPISODES_ANALYSIS
            EVENTS_ANALYSIS_RISK
            EVENTS_ANALYSIS_CTRL
            NPTS_CENSOR_NOEVENTS
            MINDAYS_POSTENR_EXPOSURE
            MAXDAYS_POSTENR_EXPOSURE
            MINDAYS_EVENT_ANALYSIS
            MAXDAYS_EVENT_ANALYSIS;
        class group type &class.;
        output out=&outfile.(drop=_:) sum(NPTS_EXPOSURE
                                          NPTS_CENSOR_ELIG 
                                          NPTS_CENSOR_DTH
                                          NPTS_ANALYSIS
                                          EPISODES_EXPOSURE
                                          EPISODES_ANALYSIS
                                          EVENTS_ANALYSIS_RISK
                                          EVENTS_ANALYSIS_CTRL
                                          NPTS_CENSOR_NOEVENTS)=
                                      	  Min(MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS)=MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS
                                          Max(MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS )=MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS;             
        run;

		*One record per episode for the class stratification;
        proc means data=_mstr nway noprint missing;
        var EPISODES_CENSOR_ELIG
			EPISODES_CENSOR_DTH
			EPISODES_CENSOR_NOEVENTS;

        class group type &class.;
        output out=&outfile.2(drop=_:) sum(EPISODES_CENSOR_ELIG
										   EPISODES_CENSOR_DTH
										   EPISODES_CENSOR_NOEVENTS)=;     
        run;

        *Store patient nums to DPLocal;
		data &outfile.;
		merge &outfile. &outfile.2;
		by group type &class.;
		run;

        %IF %length(&class.)=0 %THEN %DO;
            data DPLOCAL.&RUNID._NumCounts;
            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
            set &outfile.;
            format level $3.;
            Level="&level.";
            run;
        %END;
        %ELSE %DO;
            data DPLOCAL.&RUNID._NumCounts;
            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
            set DPLOCAL.&RUNID._NumCounts
                &outfile.(in=a);
                if a then Level="&level.";
            run;
        %END;

    %mend;

    *Overall;
    %createNumTableEntries(class=,outfile=_ALL,level=000);

    *Base Tables;
    %createNumTableEntries(class=year,outfile=_Y,Level=001);
    %createNumTableEntries(class=Sex,outfile=_S,Level=002);
    %createNumTableEntries(class=AgeGroup agegroupsort,outfile=_AG,Level=003);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort,outfile=_SAG,Level=004);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort year,outfile=_SAGY,Level=005);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort year month,outfile=_SAGYM,Level=006);
    %createNumTableEntries(class=AgeGroup agegroupsort year,outfile=_AGY,Level=007);
    %createNumTableEntries(class=AgeGroup agegroupsort year month,outfile=_AGYM,Level=008);
    %createNumTableEntries(class=Sex year,outfile=_SY,Level=009);
    %createNumTableEntries(class=Sex year month,outfile=_SYM,Level=010);
	%createNumTableEntries(class=year month,outfile=_YM,Level=011);

    *TTE_VALUE Tables;
    %createNumTableEntries(class=year TTE_VALUE,outfile=_Y_TTE,Level=401);
    %createNumTableEntries(class=Sex TTE_VALUE,outfile=_S_TTE,Level=402);
    %createNumTableEntries(class=AgeGroup agegroupsort TTE_VALUE,outfile=_AG_TTE,Level=403);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort TTE_VALUE,outfile=_SAG_TTE,Level=404);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort year TTE_VALUE,outfile=_SAGY_TTE,Level=405);
    %createNumTableEntries(class=Sex AgeGroup agegroupsort year month TTE_VALUE,outfile=_SAGYM_TTE,Level=406);
    %createNumTableEntries(class=AgeGroup agegroupsort year TTE_VALUE,outfile=_AGY_TTE,Level=407);
    %createNumTableEntries(class=AgeGroup agegroupsort year month TTE_VALUE,outfile=_AGYM_TTE,Level=408);
    %createNumTableEntries(class=Sex year TTE_VALUE,outfile=_SY_TTE,Level=409);
    %createNumTableEntries(class=Sex year month TTE_VALUE,outfile=_SYM_TTE,Level=410);
	%createNumTableEntries(class=year month TTE_VALUE,outfile=_YM_TTE,Level=411);
    %createNumTableEntries(class=TTE_VALUE,outfile=_TTE,Level=412);


	proc sort data=DPLOCAL.&RUNID._NumCounts;
	by group Level sex agegroupsort year month;
	run; 

/* TO BE REACTIVATED LATER WHEN DENOMS ARE AVAILABLE...
proc sort data=DPLOCAL.&RUNID._DenomCounts;
by group Level sex agegroupsort year month;
run; 

data _t3_CIDA;
merge DPLOCAL.&RUNID._NumCounts (in=a);
by group Level sex agegroupsort year month;

if not b and level=:"0" then do;
DenNumPts=0;
DenNumMemDays=0;
end;

if b and level=:"0" and missing(DenNumPts) then do;
DenNumPts=0;
DenNumMemDays=0;
end;
run;
*/

	proc sql noprint;
	create table MSOC.&RUNID._t3_CIDA as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		month,
            TTE_VALUE,
            NPTS_EXPOSURE,
            NPTS_CENSOR_ELIG, 
            NPTS_CENSOR_DTH,
            NPTS_CENSOR_NOEVENTS,
            NPTS_ANALYSIS,
            EPISODES_EXPOSURE,
            EPISODES_ANALYSIS,
            EVENTS_ANALYSIS_RISK,
            EVENTS_ANALYSIS_CTRL,
            MINDAYS_EVENT_ANALYSIS,
            MAXDAYS_EVENT_ANALYSIS,
            MINDAYS_POSTENR_EXPOSURE,
            MAXDAYS_POSTENR_EXPOSURE,
			EPISODES_CENSOR_ELIG,
			EPISODES_CENSOR_DTH,
			EPISODES_CENSOR_NOEVENTS
    from DPLOCAL.&RUNID._NumCounts;
	quit;

	proc datasets library=work nolist nowarn;
	delete &RUNID._mstr;
	quit;

%mend ms_createTablesT3NoCensor;

%macro ms_createTablesT3Censor();

	/****************************************************************/
    /* Squaring (or making sure all possible values are represented */
    /****************************************************************/
    data _AgeGroupYearMonthSex;
    merge _cohortfile
          _TYPE3FILE(keep=group T3CtrlFrom T3CtrlTo T3RiskFrom T3RiskTo);
    by group;
    Type=3;
    format AgeGroup $9.;
    numgrp = countw("&AGESTRAT."," ") ;

    *Sumarization Vars;
	NPTS_EXPOSURE=0;
	NPTS_CENSOR_ELIG=0; 
	NPTS_CENSOR_DTH=0;
	NPTS_CENSOR_NOEVENTS=0;
	NPTS_ANALYSIS=0;
	EPISODES_EXPOSURE=0;
	EPISODES_ANALYSIS=0;
	EVENTS_ANALYSIS_RISK=0;
	EVENTS_ANALYSIS_CTRL=0;
	MINDAYS_EVENT_ANALYSIS=0;
	MAXDAYS_EVENT_ANALYSIS=0;
	MINDAYS_POSTENR_EXPOSURE=0;
	MAXDAYS_POSTENR_EXPOSURE=0;	
	EPISODES_CENSOR_ELIG=0;
	EPISODES_CENSOR_DTH=0;
	EPISODES_CENSOR_NOEVENTS=0;

    do i=1 to numgrp;
        AgeGroup=scan("&AGESTRAT",i," ");
        AgeGroupSort=i;
        do per=1 to &nummonths.;
            *create year month variables;
            date=intnx('month',&startfollowup.,per-1,'begin');
            month=month(date);
            year=year(date);
            do TTE_VALUE=T3CtrlFrom to T3CtrlTo;
TTC_VALUE=TTE_VALUE;
                    sex="F";output;
                    sex="M";output;
                    sex="U";output;
                    sex="A";output;
            end;
            do TTE_VALUE=T3RiskFrom to T3RiskTo;
TTC_VALUE=TTE_VALUE;
                    sex="F";output;
                    sex="M";output;
                    sex="U";output;
                    sex="A";output;
            end;
        end;
    end;
    keep group Type AgeGroup AgeGroupSort sex year month NPTS_: EPISODES_: EVENTS_: MINDAYS_: MAXDAYS_: TTE_VALUE 
TTC_VALUE;
    run;

    proc sql noprint;
    create table _mstr as 
    select mstr.*,
           age.AgeGroupSort
    from  DPLocal.&RUNID._mstr_Censor as mstr left join
          _age as age
    on age.AgeGroup=mstr.AgeGroup;
    quit;

    /**********************/
    /* Compute Numerators */
    /**********************/

    data _mstr;
    set _mstr(in=a)
        _AgeGroupYearMonthSex(in=b);
    if a then do;
        month=month(indexdt);
        NPTS_EXPOSURE=1;
        NPTS_CENSOR_ELIG=CensorEnrol; 
        NPTS_CENSOR_DTH=CensorDeath;
        NPTS_ANALYSIS=AnalysisCohort;
		EPISODES_CENSOR_ELIG=CensorEnrol;
		EPISODES_CENSOR_DTH=CensorDeath;		
		EPISODES_EXPOSURE=Pt_Exposure_Episode_Num>0;
		EPISODES_ANALYSIS=(Pt_Analysis_Episode_Num>0 and WinOfevent ne "");
        EVENTS_ANALYSIS_RISK=(WinOfEvent="RISK");
        EVENTS_ANALYSIS_CTRL=(WinOfEvent="CTRL");

        NPTS_CENSOR_NOEVENTS=max(0,AnalysisCohort,CensorDeath,CensorEnrol,EVENTS_ANALYSIS_RISK,EVENTS_ANALYSIS_CTRL)=0;
		EPISODES_CENSOR_NOEVENTS=NPTS_CENSOR_NOEVENTS;

        TTE_VALUE=DAYS_FROM_EXPOS_TO_EVENT;
TTC_VALUE=TTC;
		patient=1;
		/*already calculated in cidanum*/
		/*      
		        Pt_Exposure_Episode_Num
		        Pt_Analysis_Episode_Num
		        DAYS_FROM_EXPOS_TO_RISK_EVENT;
		        DAYS_FROM_EXPOS_TO_CTRL_EVENT;
		        DAYS_FROM_EXPOS_TO_EVENT;*/

    end;
    run;

    %macro createNumTableEntriesCensor(class=,outfile=,level=);

        *One record per patient for the class stratification;
        proc means data=_mstr nway noprint missing;
        var NPTS_EXPOSURE
            NPTS_CENSOR_ELIG 
            NPTS_CENSOR_DTH
            NPTS_ANALYSIS
            EPISODES_EXPOSURE
            EPISODES_ANALYSIS
            EVENTS_ANALYSIS_RISK
            EVENTS_ANALYSIS_CTRL
            NPTS_CENSOR_NOEVENTS
            POSTENR_EXPOSURE
			DAYS_FROM_EXPOS_TO_EVENT;

        class group type PatId &class.;
        id patient;
        output out=_num0(drop=_:) max(NPTS_EXPOSURE)=NPTS_EXPOSURE
                                  max(NPTS_CENSOR_ELIG)=NPTS_CENSOR_ELIG 
                                  max(NPTS_CENSOR_DTH)=NPTS_CENSOR_DTH
                                  max(NPTS_ANALYSIS)=NPTS_ANALYSIS
                                  sum(EPISODES_EXPOSURE)=EPISODES_EXPOSURE
                                  sum(EPISODES_ANALYSIS)=EPISODES_ANALYSIS
                                  sum(EVENTS_ANALYSIS_RISK)=EVENTS_ANALYSIS_RISK
                                  sum(EVENTS_ANALYSIS_CTRL)=EVENTS_ANALYSIS_CTRL
                                  max(NPTS_CENSOR_NOEVENTS)=NPTS_CENSOR_NOEVENTS
                                  min(POSTENR_EXPOSURE)=MINDAYS_POSTENR_EXPOSURE
                                  max(POSTENR_EXPOSURE)=MAXDAYS_POSTENR_EXPOSURE
                                  min(DAYS_FROM_EXPOS_TO_EVENT)=MINDAYS_EVENT_ANALYSIS
                                  max(DAYS_FROM_EXPOS_TO_EVENT)=MAXDAYS_EVENT_ANALYSIS;
        run;

        *One record per patient for the class stratification;
        proc means data=_num0 missing nway noprint;
        var NPTS_EXPOSURE
            NPTS_CENSOR_ELIG 
            NPTS_CENSOR_DTH
            NPTS_ANALYSIS
            EPISODES_EXPOSURE
            EPISODES_ANALYSIS
            EVENTS_ANALYSIS_RISK
            EVENTS_ANALYSIS_CTRL
            NPTS_CENSOR_NOEVENTS
            MINDAYS_POSTENR_EXPOSURE
            MAXDAYS_POSTENR_EXPOSURE
            MINDAYS_EVENT_ANALYSIS
            MAXDAYS_EVENT_ANALYSIS;
        class group type &class.;
        output out=&outfile.(drop=_:) sum(NPTS_EXPOSURE
                                          NPTS_CENSOR_ELIG 
                                          NPTS_CENSOR_DTH
                                          NPTS_ANALYSIS
                                          EPISODES_EXPOSURE
                                          EPISODES_ANALYSIS
                                          EVENTS_ANALYSIS_RISK
                                          EVENTS_ANALYSIS_CTRL
                                          NPTS_CENSOR_NOEVENTS)=
                                      	  Min(MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS)=MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS
                                          Max(MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS )=MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS;             
        run;

		*One record per episode for the class stratification;
        proc means data=_mstr nway noprint missing;
        var EPISODES_CENSOR_ELIG
			EPISODES_CENSOR_DTH
			EPISODES_CENSOR_NOEVENTS;

        class group type &class.;
        output out=&outfile.2(drop=_:) sum(EPISODES_CENSOR_ELIG
										   EPISODES_CENSOR_DTH
										   EPISODES_CENSOR_NOEVENTS)=;     
        run;

        *Store patient nums to DPLocal;
		data &outfile.;
		merge &outfile. &outfile.2;
		by group type &class.;
		run;

        %IF %length(&class.)=0 %THEN %DO;
            data DPLOCAL.&RUNID._NumCounts_Censor;
            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
            set &outfile.;
            format level $3.;
            Level="&level.";
            run;
        %END;
        %ELSE %DO;
            data DPLOCAL.&RUNID._NumCounts_Censor;
            *retain group sex agegroupsort year month DenNumPts DenNumMemDays;
            set DPLOCAL.&RUNID._NumCounts_Censor
                &outfile.(in=a);
                if a then Level="&level.";
            run;
        %END;

    %mend;

    *Overall;
    %createNumTableEntriesCensor(class=,outfile=_ALL,level=000);

    *Base Tables;
    %createNumTableEntriesCensor(class=year,outfile=_Y,Level=001);
    %createNumTableEntriesCensor(class=Sex,outfile=_S,Level=002);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort,outfile=_AG,Level=003);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort,outfile=_SAG,Level=004);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year,outfile=_SAGY,Level=005);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year month,outfile=_SAGYM,Level=006);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year,outfile=_AGY,Level=007);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year month,outfile=_AGYM,Level=008);
    %createNumTableEntriesCensor(class=Sex year,outfile=_SY,Level=009);
    %createNumTableEntriesCensor(class=Sex year month,outfile=_SYM,Level=010);
	%createNumTableEntriesCensor(class=year month,outfile=_YM,Level=011);

    *TTE_VALUE Tables;
    %createNumTableEntriesCensor(class=year TTE_VALUE,outfile=_Y_TTE,Level=401);
    %createNumTableEntriesCensor(class=Sex TTE_VALUE,outfile=_S_TTE,Level=402);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort TTE_VALUE,outfile=_AG_TTE,Level=403);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort TTE_VALUE,outfile=_SAG_TTE,Level=404);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year TTE_VALUE,outfile=_SAGY_TTE,Level=405);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year month TTE_VALUE,outfile=_SAGYM_TTE,Level=406);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year TTE_VALUE,outfile=_AGY_TTE,Level=407);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year month TTE_VALUE,outfile=_AGYM_TTE,Level=408);
    %createNumTableEntriesCensor(class=Sex year TTE_VALUE,outfile=_SY_TTE,Level=409);
    %createNumTableEntriesCensor(class=Sex year month TTE_VALUE,outfile=_SYM_TTE,Level=410);
	%createNumTableEntriesCensor(class=year month TTE_VALUE,outfile=_YM_TTE,Level=411);
    %createNumTableEntriesCensor(class=TTE_VALUE,outfile=_TTE,Level=412);

	*TTC_VALUE Tables;
    %createNumTableEntriesCensor(class=year TTE_VALUE TTC_VALUE,outfile=_Y_TTE,Level=901);
    %createNumTableEntriesCensor(class=Sex TTE_VALUE TTC_VALUE,outfile=_S_TTE,Level=902);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort TTE_VALUE TTC_VALUE,outfile=_AG_TTE,Level=903);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort TTE_VALUE TTC_VALUE,outfile=_SAG_TTE,Level=904);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year TTE_VALUE TTC_VALUE,outfile=_SAGY_TTE,Level=905);
    %createNumTableEntriesCensor(class=Sex AgeGroup agegroupsort year month TTE_VALUE TTC_VALUE,outfile=_SAGYM_TTE,Level=906);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year TTE_VALUE TTC_VALUE,outfile=_AGY_TTE,Level=907);
    %createNumTableEntriesCensor(class=AgeGroup agegroupsort year month TTE_VALUE TTC_VALUE,outfile=_AGYM_TTE,Level=908);
    %createNumTableEntriesCensor(class=Sex year TTE_VALUE TTC_VALUE,outfile=_SY_TTE,Level=909);
    %createNumTableEntriesCensor(class=Sex year month TTE_VALUE TTC_VALUE,outfile=_SYM_TTE,Level=910);
	%createNumTableEntriesCensor(class=year month TTE_VALUE TTC_VALUE,outfile=_YM_TTE,Level=911);
    %createNumTableEntriesCensor(class=TTE_VALUE TTC_VALUE,outfile=_TTE,Level=912);
	%createNumTableEntriesCensor(class=TTC_VALUE,outfile=_TTC,Level=913);

	proc sort data=DPLOCAL.&RUNID._NumCounts_Censor;
	by group Level sex agegroupsort year month;
	run; 

	proc sql noprint;
	create table MSOC.&RUNID._t3_CIDA_Censor as
	select group,
    		Level,
    		sex,
    		agegroup,
    		agegroupsort as agegroupnum,
    		year,
    		month,
            TTE_VALUE,
			TTC_VALUE,
            NPTS_EXPOSURE,          
            NPTS_ANALYSIS,
            EPISODES_EXPOSURE,
            EPISODES_ANALYSIS,
            EVENTS_ANALYSIS_RISK,
            EVENTS_ANALYSIS_CTRL,
            MINDAYS_EVENT_ANALYSIS,
            MAXDAYS_EVENT_ANALYSIS,
            MINDAYS_POSTENR_EXPOSURE,
            MAXDAYS_POSTENR_EXPOSURE			
    from DPLOCAL.&RUNID._NumCounts_Censor;
	quit;

	proc datasets library=work nolist nowarn;
	delete _: &RUNID._mstr;
	quit;

%mend ms_createTablesT3Censor;

%ms_createTablesT3NoCensor();

%ms_createTablesT3Censor();

%put NOTE: ******** END OF MACRO: ms_cidatablest3  v1.6 ********;

%mend ms_cidatablest3;
