****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createclaimepi.sas  
*
* Created (mm/dd/yyyy): 07/31/2014
* Last modified: 07/31/2014
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  	This program creates episodes of claims with a given tolerence gap between two consecutive 
*  	claims.
*
*  Program inputs:                                                                                   
*	-Dataset containing the claims to create episodes with                                   
*  Program outputs:                                                                                                                                       
*	-Dataset containing the created episodes
 
* PARAMETERS:
*  	-infile	= Dataset containing the claims to create episodes with
*  	-gap	= Maximum allowable gap between two claim (based on  Adate Expiredt and EnrStart)
*  	-outfile= Dataset containing the episodes
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;


%macro ms_createclaimepi(infile=, gap=, outfile=);

%put =====> MACRO CALLED: ms_createclaimepi v1.0;

    Proc sort data=&infile. out=&outfile.;
    by Patid Adate Expiredt;
    run;

    data &outfile.;
    set &outfile.;
    by Patid;
    
   if first.PatId then do;
      LRunOutDate=Expiredt;
      LEnrStart=Enr_Start; 
      episode=1;
      gap=.;
   end;
   else do; 
      gap = Adate - LRunOutDate-1;         
      if gap > &Gap. or lEnrStart ne Enr_Start then do;
          episode=episode+1; 
          LRunOutDate = Expiredt;
      end;
      else LRunOutDate = max(Expiredt,LRunOutDate);
   end;
   
   Group = "&ITGROUP.";   

   format LRunOutDate LEnrStart date9.;
 
    LEnrStart = Enr_Start;

   retain episode LRunOutDate LEnrStart;
run;

%put NOTE: ********END OF MACRO: ms_createclaimepi v1.0********;

%mend ms_createclaimepi;

