****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createclaimepibycat.sas  
*
* Created (mm/dd/yyyy): 09/16/2015
* Last modified: 10/22/15
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  	This program creates episodes of claims with a given tolerence gap between two consecutive 
*  	claims. Episodes are created by specific grouping category.
*
*  Program inputs:                                                                                   
*	-Dataset containing the claims to create episodes with                                   
*  Program outputs:                                                                                                                                       
*	-Dataset containing the created episodes
 
* PARAMETERS:
*  	-infile	    = Dataset containing the claims to create episodes with
*	-categories = Variables by which the episodes should be grouped when they are created
*   -lastCat	= Last variable in the categories variables
*  	-gap	    = Maximum allowable gap between two claim (based on  Adate Expiredt and EnrStart)
*  	-outfile    = Dataset containing the episodes
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*     1.1     10/22/15		DM		Changed the code to allow grouping by more than one category
*
***************************************************************************************************;


%macro ms_createclaimepibycat(infile=, category=, lastCat=, gap=, outfile=);

	%put =====> MACRO CALLED: ms_createclaimepibycat v1.1;

    Proc sort data=&infile. out=&outfile.;
    by Patid &Category. Adate Expiredt;
    run;

    data &outfile.;
    set &outfile.;
    by Patid &Category.;
    
	if first.&LastCat. then do;
	  LRunOutDate=Expiredt;
	  LEnrStart=Enr_Start; 
	  episode=1;
	  gap=.;
	end;
	else do; 
	  gap = Adate - LRunOutDate-1;         
	  if gap > &Gap. or lEnrStart ne Enr_Start then do;
	      episode=episode+1; 
	      LRunOutDate = Expiredt;
	  end;
	  else LRunOutDate = max(Expiredt,LRunOutDate);
	end;

	Group = "&ITGROUP.";   

	format LRunOutDate LEnrStart date9.;

	LEnrStart = Enr_Start;

	retain episode LRunOutDate LEnrStart;
	run;

	%put NOTE: ********END OF MACRO: ms_createclaimepibycat v1.1********;

%mend ms_createclaimepibycat;

