****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: runCIDA.sas  
*
* Created (mm/dd/yyyy): 01/26/2016
* Last modified: 
* Version: 3.3.1_PregPSMRADAR
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   Macro to run CIDA and Analytic Code if necessary.                                        
*   
*  Program inputs:                                                                                   
*                                     
*  Program outputs:                                                                                                                                       
*                                                                                                  
*  PARAMETERS:    
*		All the parameters are set by the run_programs program. 
*            
*  Programming Notes:                                                                                
*		-The DEBUG macro variable, when set to 1, allows to output working datasets in a debugging folder.                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;
/* Macro to run CIDA and Analytic Code if necessary */

%macro runCIDA ;

  proc printto log="&MSOC.&RUNID._CIDA.log" new;
  run;

  %put =====> MACRO CALLED: runCIDA v3.3.1_PregPSMRADAR;

  %let Ver=3.3.1_PregPSMRADAR;

  *Setting this variable to 1 will make the program copy files in temp folders for debugging purposes;
  %let DEBUG=0;

  /*option required to ensure successful multiple runs of CIDA*/
  options nosyntaxcheck;


  %ms_cidanum(RUNID=&RUNID.,
              MSPROJID=&MSPROJID.,  
              MSWPTYPE=&MSWPTYPE.,
              MSWPID=&MSWPID.,
			  MSDPID=&MSDPID.,
              MSVERID=&MSVERID.,
              AGESTRAT=&AGESTRAT.,
              PERIODIDSTART=&PERIODIDSTART.,
              PERIODIDEND=&PERIODIDEND.,
              ANALYSIS=&ANALYSIS.,
              MONITORINGFILE=&MONITORINGFILE.,
              COHORTFILE=&COHORTFILE.,
              TYPE1FILE=&TYPE1FILE.,
              TYPE2FILE=&TYPE2FILE.,
              TYPE3FILE=&TYPE3FILE.,
			  TYPE4FILE=&TYPE4FILE.,
			  T3METADATA=&T3METADATA.,
              COHORTCODES=&COHORTCODES.,
              INCLUSIONCODES=&INCLUSIONCODES.,
              COVARIATECODES=&COVARIATECODES.,
              STOCKPILINGFILE=&STOCKPILINGFILE.,
              UTILFILE=&UTILFILE.,
              COMBOFILE=&COMBOFILE.,
              COMORBFILE=&COMORBFILE.,
              DRUGCLASSFILE=&DRUGCLASSFILE.,
              FREEZEDATA=&FREEZEDATA.,
              LABSCODEMAP=&LABSCODEMAP.,
			  SURVEILLANCEMODE=&SURVEILLANCEMODE.,
			  PREGCODES=&PREGCODES.,
			  MOICODES=&MOICODES. ,
			  ZIPFILE=&ZIPFILE.
              );  

  %if (%UPCASE("&ANALYSIS.") eq "PS") %then %do ;
  
    %MS_COVARIATEADJUSTMENT(INDATA=DPLOCAL.&RUNID._ads_psmatch,
                            PERIODSTART=&PERIODIDSTART.,
                         	PERIODEND=&PERIODIDEND.,
                         	COVARIATECONDITION=&COVARIATECODES. ,
                         	COVARIATES_CONSIDERED=&COVARIATES_CONSIDERED.,
                         	COVARIATES_SELECTED=&COVARIATES_SELECTED.,
                         	RANKING=&RANKING.,
                         	ZERO_CELL_CORR=&ZERO_CELL_CORR.,
                         	HDVARSEL_INPUT=&RUNID._claims_ICDDX09 &RUNID._claims_ICDDX10 &RUNID._claims_DRUGCLASS &RUNID._claims_CPT &RUNID._claims_ICDPX09 &RUNID._claims_ICDPX10 &RUNID._claims_HCPCS,                         	
                         	COMPARE_INPUT=&COMPARISON. ,
                         	TYPE2FILE=&TYPE2FILE.,
						 	ANALYTICSUBGROUPS=&ANALYTICSUBGROUPS.,
                         	INDLEVEL=&INDLEVEL.,
                            PERCENTILES=&PERCENTILES.,
                            UNCONDITIONAL=&UNCONDITIONAL.,
                            DIAGNOSTICS=&DIAGNOSTICS.
                         	) ;
  %end ;

  %put NOTE: ******** END OF MACRO: runCIDA v3.3.1_PregPSMRADAR ********;

  proc printto;
  run;

%mend runCIDA ;

%runCIDA ;
